/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class beecrypt_provider_RSAKeyPairGenerator */

#ifndef _Included_beecrypt_provider_RSAKeyPairGenerator
#define _Included_beecrypt_provider_RSAKeyPairGenerator
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     beecrypt_provider_RSAKeyPairGenerator
 * Method:    generate
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_beecrypt_provider_RSAKeyPairGenerator_generate
  (JNIEnv *, jobject);

#ifdef __cplusplus
}
#endif
#endif
