/*
 * Copyright (c) 2004 Beeyond Software Holding BV
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#define BEECRYPT_CXX_DLL_EXPORT

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "beecrypt/c++/beeyond/BeeCertPathParameters.h"
#include "beecrypt/c++/beeyond/BeeCertificate.h"

#include "beecrypt/c++/util/ArrayList.h"

using namespace beecrypt::beeyond;

BeeCertPathParameters::BeeCertPathParameters()
{
}

BeeCertPathParameters::BeeCertPathParameters(KeyStore& keystore) throw (KeyStoreException, InvalidAlgorithmParameterException)
{
	Enumeration<const String>* aliases = keystore.aliases();

	while (aliases->hasMoreElements())
	{
		const String* alias = aliases->nextElement();

		if (keystore.isCertificateEntry(*alias))
		{
			// only add BeeCertificates
			const BeeCertificate* beecert = dynamic_cast<const BeeCertificate*>(keystore.getCertificate(*alias));

			if (beecert)
				_cert.add(beecert->clone());
		}
	}

	delete aliases;

	if (_cert.size() == 0)
		throw InvalidAlgorithmParameterException("KeyStore doesn't contain any trusted BeeCertificates");
}

const List<Certificate>& BeeCertPathParameters::getTrustedCertificates() const
{
	return _cert;
}

BeeCertPathParameters* BeeCertPathParameters::clone() const throw ()
{
	BeeCertPathParameters* tmp = new BeeCertPathParameters();

	tmp->_cert.addAll(_cert);

	return tmp;
}
