/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class beecrypt_provider_AES */

#ifndef _Included_beecrypt_provider_AES
#define _Included_beecrypt_provider_AES
#ifdef __cplusplus
extern "C" {
#endif
#undef beecrypt_provider_AES_MODE_ECB
#define beecrypt_provider_AES_MODE_ECB 0L
#undef beecrypt_provider_AES_MODE_CBC
#define beecrypt_provider_AES_MODE_CBC 1L
#undef beecrypt_provider_AES_MODE_CTR
#define beecrypt_provider_AES_MODE_CTR 2L
#undef beecrypt_provider_AES_PADDING_NONE
#define beecrypt_provider_AES_PADDING_NONE 0L
#undef beecrypt_provider_AES_PADDING_PKCS5
#define beecrypt_provider_AES_PADDING_PKCS5 1L
/*
 * Class:     beecrypt_provider_AES
 * Method:    allocParam
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_beecrypt_provider_AES_allocParam
  (JNIEnv *, jclass);

/*
 * Class:     beecrypt_provider_AES
 * Method:    freeParam
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_beecrypt_provider_AES_freeParam
  (JNIEnv *, jclass, jlong);

/*
 * Class:     beecrypt_provider_AES
 * Method:    init
 * Signature: (JI[B[B)V
 */
JNIEXPORT void JNICALL Java_beecrypt_provider_AES_init
  (JNIEnv *, jclass, jlong, jint, jbyteArray, jbyteArray);

/*
 * Class:     beecrypt_provider_AES
 * Method:    process
 * Signature: (J[BII[BI)I
 */
JNIEXPORT jint JNICALL Java_beecrypt_provider_AES_process
  (JNIEnv *, jclass, jlong, jbyteArray, jint, jint, jbyteArray, jint);

#ifdef __cplusplus
}
#endif
#endif
